<?php
namespace Teknisa\EST\Modules\Ajuste\Services;

use Teknisa\EST\Services\ValidacoesEstoqueService;
use \Estoque\Entities\Lanctoestoq;
use \Estoque\Entities\Itlanctoest;
use \Estoque\Entities\Posestdia;
use \Geral\Entities\Paramfilial;
use \Geral\Entities\Prodfili;
use Teknisa\Libs\Util\DateUtil;
use Zeedhi\Framework\DTO\Request\Row;

class LanctoestoqAjusteService {

    protected $dataSourceName = "lanctoestoq_ajuste";

    protected $validacoesEstoqueService;
    protected $environment;

    public function __construct (ValidacoesEstoqueService $validacoesEstoqueService, $factory, $entityManager, $environment)
    {
        $this->validacoesEstoqueService = $validacoesEstoqueService;
        $this->factory = $factory;
        $this->entityManager = $entityManager;
        $this->environment = $environment;
    }

    public function deleteLancto($cdfilial,$nrlancestq) {
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            $nrorg = $this->environment->getNrOrgTrab();

            $itens = Itlanctoest::findByCdfilialAndNrlancestqAndNrorg(
                $cdfilial, $nrlancestq, $nrorg
            );

            $lancamento = Lanctoestoq::findOneByCdfilialAndNrlancestqAndNrorg(
                $cdfilial, $nrlancestq, $nrorg
            );

            foreach($itens as $item){
                $this->entityManager->remove($item);
                $this->entityManager->flush($item);
                if($item->getIdevolestoq() =='S')
                    $this->validacoesEstoqueService->atualizaEstoque($cdfilial,$nrlancestq,true,$item->getNrsequitem());
            }
            $this->entityManager->remove($lancamento);
            $this->entityManager->flush($lancamento);
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }
    
    public function buildObjectTheCode($code, $params, $boolImplantacao = false,&$itensWithErrors,$ErrorRowNumber) {
        $pointer = 0;
        $objectRead = [];
        $boolUsaSeparador = $params['IDTPSEPARADOR'] <> 2;
        
        $data = DateUtil::getDataDeString(substr($code, $pointer, strlen($params["DSFORMDATA"])), $params["DSFORMDATA"]);
        if(is_null($data) || $data == false){
            $data = \DateTime::createFromFormat('dmY', substr($code, $pointer, strlen($params["DSFORMDATA"])));
        }
        if(!$data){
            $message = "Formato de Data Inválida ou Parametrização Invalida";
            $this->buildErrorRow($itensWithErrors,$ErrorRowNumber,$message);
        }
        $data = $data->format(Dateutil::FORMATO_BRASILEIRO);
        
        $objectRead["DTLANCESTQ"] = $data; //Lê a data
        $pointer += strlen($params["DSFORMDATA"]);
        
        //Lê o campo que identifica o produto
        $objectRead[$params["PRODUTO"]] = substr($code, $pointer, $params["QTDIGPRODUTO"]);
        $pointer += $params["QTDIGPRODUTO"];

        //Lê a quantidade (inteira)
        $int = substr($code, $pointer, $params["QTNUMEROINT"]);
        $pointer += $params["QTNUMEROINT"];
        //Confere se usa separador decimal
        $separador = substr($code, $pointer, 1);
        $message = "Separador informado ( $separador ) não corresponde com a parametrização definida.";
        if($boolUsaSeparador){
            if(($separador != "," && $params["IDTPSEPARADOR"] == 0) || ($separador != "." && $params["IDTPSEPARADOR"] == 1)){
                $this->buildErrorRow($itensWithErrors,$ErrorRowNumber,$message);
            }
            $pointer += 1;
        }else{
            if(($separador == "," || $separador == ".") && ($params["IDTPSEPARADOR"] == 2)){
                $this->buildErrorRow($itensWithErrors,$ErrorRowNumber,$message);
            }
            $pointer += 0;
        }
        //Lê a quantidade (decimal)
        $decimal = substr($code, $pointer, $params["QTNUMERODEC"]);
        $pointer += $params["QTNUMERODEC"];
        
        //Seta Quantidade
        $objectRead["QTTOTLANCTO"] = $int || $decimal ? floatval($int . '.' . $decimal) : 0;

        if($boolImplantacao){
            //Lê o valor (inteiro)
            $intVr = substr($code, $pointer, $params["QTNUMEROINT"]);
            $pointer += $params["QTNUMEROINT"];
            //Confere se usa separador decimal
            $pointer += $boolUsaSeparador? 1 : 0;
            //Lê a valor (decimal)
            $decimalVr = substr($code, $pointer, $params["QTNUMERODEC"]);
            $pointer += $params["QTNUMERODEC"];
            //Seta Quantidade
            $objectRead["VRUNILANCTO"] = $int || $decimal ? floatval($intVr . '.' . $decimalVr) : 0;
            $objectRead["VRUNITBRUT"] = $objectRead["VRUNILANCTO"];
            
        }
            
        //Lê o almoxarficado
        $objectRead["CDALMOXARIFE"] = substr($code, $pointer, $params["QTTAMALMOX"]);
        $pointer += $params["QTTAMALMOX"];
        //Lê o localização
        $objectRead["CDLOCALESTOQ"] = substr($code, $pointer, $params["QTTAMLOCAL"]);
        $pointer += $params["QTTAMLOCAL"];

        //Lê o lote
        $objectRead["NRLOTEESTQ"] = substr($code, $pointer, $params["QTTAMLOTE"]);
        $pointer += $params["QTTAMLOTE"];

        //Lê o sublote
        $objectRead["NRSUBLOTE"] = substr($code, $pointer, $params["QTTAMSUBLOTE"]);
        $pointer += $params["QTTAMSUBLOTE"];
        
        if($boolImplantacao){
            //Lê a data de validade do Lote
            $dataValidade = DateUtil::getDataDeString(substr($code, $pointer, strlen($params["DSFORMDATA"])), $params["DSFORMDATA"]);
            if(is_null($dataValidade) || $dataValidade == false){
                $dataValidade = \DateTime::createFromFormat('dmY', substr($code, $pointer, strlen($params["DSFORMDATA"])));
            }
            if(!$dataValidade){
                $dataValidade = null;
            }else{
                $dataValidade = $dataValidade->format(Dateutil::FORMATO_BRASILEIRO);
            }
            
            $objectRead["DTVALILOTE"] = $dataValidade; 
            $pointer += strlen($params["DSFORMDATA"]);
            
            
            //Lê a data de fabricacao
            $dataFabric = DateUtil::getDataDeString(substr($code, $pointer, strlen($params["DSFORMDATA"])), $params["DSFORMDATA"]);
            if(is_null($dataFabric) || $dataFabric == false){
                $dataFabric = \DateTime::createFromFormat('dmY', substr($code, $pointer, strlen($params["DSFORMDATA"])));
            }
            if(!$dataFabric){
                $dataFabric = null;
            }else{
                $dataFabric = $dataFabric->format(Dateutil::FORMATO_BRASILEIRO);
            }
            $objectRead["DTFABRPROD"] = $dataFabric; 
        }
        
        return $objectRead;
    }
    
    public function getParamfilial($filial) {
        $nrorg = $this->environment->getNrOrgTrab();
        return Paramfilial::findOneByCdfilialAndNrorg($filial, $nrorg);
    }
    
    public function getParamCustos($filial) {
        $arrCustos = array();
        $entFilial = $this->getParamfilial($filial);
        $tipoCusto = $entFilial->getCdtipcustest();
        
        if($tipoCusto == '08' || $tipoCusto == '09'){
            $arrCustos['CUSTLIQU'] = '08';
            $arrCustos['CUSTBRUT'] = '09';
        }elseif($tipoCusto == '03' || $tipoCusto == '04'){
            $arrCustos['CUSTLIQU'] = '03';
            $arrCustos['CUSTBRUT'] = '04';
        }elseif($tipoCusto == '01' || $tipoCusto == '02'){
            $arrCustos['CUSTLIQU'] = '01';
            $arrCustos['CUSTBRUT'] = '02';
        }else{
            $arrCustos['CUSTLIQU'] = $tipoCusto;
            $arrCustos['CUSTBRUT'] = $tipoCusto;
        }
        return $arrCustos;
    }
    
    public function getCdSegMercFilial($cdFilial) {
        $resultado = $this->factory->getSegMercFilial($cdFilial);
        if(isset($resultado[0])){
            return $resultado[0]["CDSEGMER"];
        } else{
            return null;  
        }
    }
    
    public function validaCustoAjuste($filial, $cdprodesto, $dtLancamento, $cdalmoxarife = ' ', $cdlocalestoq = ' ', $nrloteestq = ' ', $nrsublote = ' ', $idOrdemClc = 'P'){
        $nrorg = $this->environment->getNrOrgTrab();
        $tipoCusto = Paramfilial::findOneByCdfilialAndNrorg($filial, $nrorg)->getCdtipcustest();
        
        if($tipoCusto == NULL)
            $tipoCusto = '03';
        
        $dtVerificaPosicao = DateUtil::adicionaIntervalo($this->formataData($dtLancamento), 1)->format(Dateutil::FORMATO_BRASILEIRO);
        
        $dtposiestq = Posestdia::getRepository()->retornaDataPosicaoEstoque($filial, $cdalmoxarife, $cdlocalestoq, $nrloteestq, $nrsublote, $cdprodesto, $dtVerificaPosicao);
        $qtde=-1;

        $dtLancamento = $this->formataData($dtLancamento);
        $dtLancamento = $dtLancamento->format(Dateutil::FORMATO_BRASILEIRO);
        $custo = Posestdia::getRepository()->retornaCustoProdutoEstoque(
            $filial, $tipoCusto, $dtLancamento, $cdalmoxarife, $cdlocalestoq, $cdprodesto, $nrloteestq, $nrsublote, $dtposiestq, $qtde
        );

        if(isset($custo[0])){
            $custo = $custo[0];
            $valorCusto = $custo['VRCUSTOPROD'];

            
            $qtdeAnte = $custo['QTESTOQDIA'];
            $qtAnterior = $custo['QTESTOQANT'];

            $valrAnte = $valorCusto *$qtdeAnte;
            $vlAnterior = $valorCusto *$qtAnterior;

            if( ($qtdeAnte <> 0) or ($qtAnterior <> 0) ){
               if( ($qtdeAnte <> 0) ){
                  $vrMedi = $valrAnte / $qtdeAnte ;
               }
               else {
                  $vrMedi = $vlAnterior /  $qtAnterior;
               }
            }
            else {
               $vrMedi = 0;
            }
            
            $validacaoCusto = array();
            $validacaoCusto['valrAnte'] = $valrAnte*$custo['VRFATOCONV'];
            $validacaoCusto['qtdeAnte'] = $qtdeAnte;
            $validacaoCusto['valrMedi'] = $vrMedi*$custo['VRFATOCONV'];
            $validacaoCusto['vlAnterior'] = $vlAnterior*$custo['VRFATOCONV'];
            return $validacaoCusto;//$vrMedi*$custo['VRFATOCONV'];
        }else{
            return 0;
        }
    }

    // Formata a data de forma que possa ser inserida no banco
    public static function formataData($data) {
        if (!$data || !is_string($data))
            return $data;

        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);

        $data = DateUtil::getDataDeString($data);
        if($data)
            $data = DateUtil::truncateData($data);
        return $data;
    }
    
    public function completaProduto($produto, $cdfilial, $cdfilimovi, $dtLancEst,&$itensWithErrors,$ErrorRowNumber){
        $produtoTabela = $this->getProdutoTabela($produto, $cdfilimovi);
        
        $codImportado = isset($produto['CDARVPROD'])? ($produto['CDARVPROD']) : (isset($produto['CDPRODUTO'])? $produto['CDPRODUTO'] : $produto['CDBARPRODUTO']);
        
        if(!$produtoTabela){
            if(empty($codImportado)){
                $message = "Filial sem parametrização Para importação";
                $this->buildErrorRow($itensWithErrors,$ErrorRowNumber,$message);
            }else{
                $message = "Produto não encontrado. Codigo: ".$codImportado;
                $this->buildErrorRow($itensWithErrors,$ErrorRowNumber,$message);
            }
        }
        
         // validações de controle de lote
        $nrorg = $this->environment->getNrOrgTrab();
        $paramfilial = Paramfilial::findOneByCdfilialAndNrorg($cdfilimovi, $nrorg);
        
        $utAlmox = $paramfilial->getIdutilalmox();
        $utLcEst = $paramfilial->getIdutillcestq();
        $utlote = $paramfilial->getIdutillote();
        
        $prodfili = Prodfili::findOneByCdprodutoAndCdfilialAndNrorg($produtoTabela["CDPRODMOVI"], $cdfilimovi, $nrorg);
        $prodUtLote  = $prodfili->getIdutctrllote();
        $prodAlmox   = $prodfili->getCdalmoxarife();
        $prodLocal   = $prodfili->getCdlocalestoq();
        
        if($produto["NRLOTEESTQ"]>0 && isset($produto["DTVALILOTE"])){
            if(!$produto["DTVALILOTE"]){
                $message = "Filial controla lote, porém data de validade não foi informada";
                $this->buildErrorRow($itensWithErrors,$ErrorRowNumber,$message);
            }else{
                if(strtotime($produto["DTVALILOTE"]) < strtotime($produto["DTFABRPROD"])){
                    $message = "Data de validade menor que a data de fabricação.";
                    $this->buildErrorRow($itensWithErrors,$ErrorRowNumber,$message);
                }
            }
        };
        
        if($utAlmox == 'S' && !$produto["CDALMOXARIFE"]){
            $message = "Filial controla almoxarifado e o mesmo não foi encontrado no arquivo.";
            $this->buildErrorRow($itensWithErrors,$ErrorRowNumber,$message);
        }
        if($utLcEst == 'S' && !$produto["CDLOCALESTOQ"]){
            $message = "Filial controla local de estoque e o mesmo não foi encontrado no arquivo.";
            $this->buildErrorRow($itensWithErrors,$ErrorRowNumber,$message);
        }
        if($utlote == 'S' && $prodUtLote == 'S' && !$produto["NRLOTEESTQ"]){
            $message = "Filial controla lote e o mesmo não foi encontrado no arquivo.";
            $this->buildErrorRow($itensWithErrors,$ErrorRowNumber,$message);
        }
        
        
        $qttotlancto = $produto["QTTOTLANCTO"] ? $produto["QTTOTLANCTO"] : $produtoTabela["QTTOTLANCTO"];
        $vrunit = isset($produto["VRUNILANCTO"]) ? $produto["VRUNILANCTO"] : $produtoTabela["VRUNILANCTO"];
        $vrunitbrut = isset($produto["VRUNITBRUT"]) ? $produto["VRUNITBRUT"] : $produtoTabela["VRUNITBRUT"];
        
        $vrtotlancto = $vrunit * $qttotlancto;
        $vrlanctobrut = $vrunitbrut * $qttotlancto;
        
        $vrfatoconv = $produtoTabela['VRFATOCONV']? $produtoTabela['VRFATOCONV'] : 1;
        $qtlanctoest = $qttotlancto * $vrfatoconv;
        
        $produtoTabela["CDFILIAL"] = $cdfilial;
        $produtoTabela["CDFILIMOVI"] = $cdfilimovi;
        $produtoTabela["DTLANCESTQ"] = $produto["DTLANCESTQ"] ? $produto["DTLANCESTQ"] : $dtLancEst;
        $produtoTabela["CDPRODUTO"] = isset($produto["CDPRODUTO"]) ? $produto["CDPRODUTO"] : $produtoTabela["CDPRODUTO"];
        $produtoTabela["CDARVPROD"] = isset($produto["CDARVPROD"]) ? $produto["CDARVPROD"] : $produtoTabela["CDARVPROD"];
        $produtoTabela["CDALMOXARIFE"] = $produto["CDALMOXARIFE"] ? $produto["CDALMOXARIFE"] : ($prodAlmox? $prodAlmox : ' ');
        $produtoTabela["CDLOCALESTOQ"] = $produto["CDLOCALESTOQ"] ? $produto["CDLOCALESTOQ"] : ($prodLocal? $prodLocal : ' ');
        $produtoTabela["NRLOTEESTQ"] = $produto["NRLOTEESTQ"] ? $produto["NRLOTEESTQ"] : $produtoTabela["NRLOTEESTQ"];
        $produtoTabela["NRSUBLOTE"] = $produto["NRSUBLOTE"] ? $produto["NRSUBLOTE"] : $produtoTabela["NRSUBLOTE"];
        $produtoTabela["QTTOTLANCTO"] = $qttotlancto;
        $produtoTabela["VRUNILANCTO"] = $vrunit;
        $produtoTabela["VRUNITBRUT"] = $vrunitbrut;
        $produtoTabela["QTLANCTOEST"] = $qtlanctoest;
        $produtoTabela["CUSTO"] = $this->validaCustoAjuste($cdfilimovi, $produtoTabela["CDPRODMOVI"], $produtoTabela["DTLANCESTQ"], $produtoTabela["CDALMOXARIFE"], $produtoTabela["CDLOCALESTOQ"], $produtoTabela["NRLOTEESTQ"], $produtoTabela["NRSUBLOTE"], null);
        $produtoTabela["DTVALILOTE"] = isset($produto["DTVALILOTE"]) ? $produto["DTVALILOTE"] : null;
        $produtoTabela["DTFABRPROD"] = isset($produto["DTFABRPROD"]) ? $produto["DTFABRPROD"] : null;

        return $produtoTabela;
    }
    
    public function getProdutoTabela($produto, $cdfilial){

        if(isset($produto["CDARVPROD"])){
            $parameters = array(
                "P_CDARVPROD" => str_replace(" ", "", $produto["CDARVPROD"])
            );
        } else if(isset($produto["CDBARPRODUTO"])){
            $parameters = array(
                "P_CDBARPRODUTO" => str_replace(" ", "", $produto["CDBARPRODUTO"])
            );
        } else{
            $parameters = array(
                "P_CDPRODUTO" => str_replace(" ", "", $produto["CDPRODUTO"])
            ); 
        }
        $parameters["P_CDFILIAL"] = $cdfilial;
        return $this->factory->getProdutoTabela($parameters);
    }
    
    public function insertLoteProduto($produto){
        $cdfilial   = $produto['CDFILIAL'];
        $cdproduto  = $produto['CDPRODUTO'];
        $nrloteestq = $produto['NRLOTEESTQ'];
        $nrsublote  = $produto['NRSUBLOTE'];
        $dtlancamento = $produto["DTLANCESTQ"];
        $dtvalilote = isset($produto['DTVALILOTE'])? $produto['DTVALILOTE'] : null;
        $dtfrabric = isset($produto['DTFABRPROD'])? $produto['DTFABRPROD'] : null;
        
        return $this->factory->insertLoteProduto($cdfilial, $cdproduto, $nrloteestq, $nrsublote, $dtlancamento, $dtvalilote, $dtfrabric);
        
    }
    
    public function insertLocaisEstoque($produto){
        $this->insertLoteProduto($produto);
        $nrorg          = $nrorg = $this->environment->getNrOrgTrab();
        $cdfilial       = $produto['CDFILIMOVI'];
        $cdalmoxarife   = isset($produto['CDALMOXARIFE'])? $produto['CDALMOXARIFE'] : ' ';
        $dsalmoxarife   = isset($produto['CDALMOXARIFE'])? $produto['CDALMOXARIFE'] : ' ';
        $this->factory->insertAlmoxarife($cdfilial, $cdalmoxarife, $dsalmoxarife, $nrorg);
        $cdlocalestoq   = isset($produto['CDLOCALESTOQ'])? $produto['CDLOCALESTOQ'] : ' ';
        $dslocalestoq   = isset($produto['CDLOCALESTOQ'])? $produto['CDLOCALESTOQ'] : ' ';
        $estdesmemb     = 'N';
        $this->factory->insertLocalEstoque($cdfilial, $cdalmoxarife, $cdlocalestoq, $dslocalestoq, $estdesmemb, $nrorg);
    }
    
    public function buildErrorRow(&$itensWithErrors,$ErrorRowNumber,$message){
        array_push($itensWithErrors,[
            "IDREF" => $ErrorRowNumber,
            "DESCERROR" => $message
        ]);
        throw new \Exception($message);
    }
    
}